import React__default, { useRef, useState, createRef, useEffect, createElement } from 'react';
import { WidthProvider, Responsive } from 'react-grid-layout';
import _ from 'lodash';
import { Dropdown } from 'primereact/dropdown';
import { Dialog } from 'primereact/dialog';
import { InputText } from 'primereact/inputtext';
import { Button } from 'primereact/button';
import axios from 'axios';
import ReactApexChart from 'react-apexcharts';
import { OverlayPanel } from 'primereact/overlaypanel';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;

  _setPrototypeOf(subClass, superClass);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };
  return _setPrototypeOf(o, p);
}

var DataService = /*#__PURE__*/function () {
  function DataService() {}

  var _proto = DataService.prototype;

  _proto.getObjectTables = function getObjectTables(filters) {
    var token = this.getTokenFromStorage();
    var apiURL = 'http://localhost:9996/api/ObjectTableViews/getbyfilters?';
    apiURL = this.ConcatFiltersToURL(filters, apiURL);
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };

  _proto.getObjectFields = function getObjectFields(filters) {
    var token = this.getTokenFromStorage();
    var apiURL = 'http://localhost:9996/api/ObjectFieldViews/getbyfilters?';
    apiURL = this.ConcatFiltersToURL(filters, apiURL);
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };

  _proto.getDataByFilters = function getDataByFilters(filters, tableName) {
    var token = this.getTokenFromStorage();
    var apiURL = "http://localhost:9996/api/" + tableName + "Views/getbyfilters?";
    apiURL = this.ConcatFiltersToURL(filters, apiURL);
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };

  _proto.ConcatFiltersToURL = function ConcatFiltersToURL(filters, urlparameters) {
    var mykeys = Object.keys(filters);
    var addtionalFiltersValues = null;

    for (var i in mykeys) {
      var propName = mykeys[i];
      var propValue = filters[propName];
      var ignoreFilter = propName.indexOf("Operator") > 0 && propValue == "Equals" || propName == "AdditionalFilters";

      if (urlparameters != "?") {
        urlparameters = urlparameters.concat('&');
      }

      if (!ignoreFilter) {
        propValue = propValue ? encodeURIComponent(propValue) : '';
        urlparameters = urlparameters.concat(propName.concat('=').concat(propValue));
      }

      if (propName == "AdditionalFilters" && propValue.length > 0) {
        addtionalFiltersValues = JSON.stringify(propValue);
      }
    }

    if (addtionalFiltersValues) {
      urlparameters = urlparameters.concat("&AdditionalFilters=").concat(addtionalFiltersValues);
    }

    return urlparameters;
  };

  _proto.getTokenFromStorage = function getTokenFromStorage() {
    return window.localStorage.getItem("token") || '';
  };

  return DataService;
}();

var ApiQueryFilters = /*#__PURE__*/function () {
  function ApiQueryFilters() {
    this.PageIndex = null;
    this.PageSize = null;
    this.SortBy = null;
    this.SortDirection = null;
    this.GetCount = null;
    this.Tenant = null;
    this.AdditionalFilters = [];
    this.ForceCacheRefresh = false;
    this.DontApplyVirtualization = false;
    this.queryId = '';
    this.queryCode = '';
    this.userid = '';
    this.ObjectTableName = '';
    this.Filter1Name = null;
    this.Filter1Operator = null;
    this.GetAll = true;
  }

  var _proto = ApiQueryFilters.prototype;

  _proto.addAdditionalFilter = function addAdditionalFilter(FieldName, FieldValue, FieldValue2, FieldValue3, Operator, IsCustom, DisplayInList, IsCustomField, FieldDataType, IgnoreFilter, IsCacheOnClient, ForceEnableAdd) {
    if (IgnoreFilter === void 0) {
      IgnoreFilter = false;
    }

    if (IsCacheOnClient === void 0) {
      IsCacheOnClient = false;
    }

    if (ForceEnableAdd === void 0) {
      ForceEnableAdd = false;
    }

    var params = new ApiQueryFiltersAddParams();
    params.FieldName = FieldName;
    params.FieldValue = FieldValue;
    params.FieldValue2 = FieldValue2;
    params.FieldValue3 = FieldValue3;
    params.Operator = Operator;
    params.IsCustom = IsCustom;
    params.DisplayInList = DisplayInList;
    params.IsCustomField = IsCustomField;
    params.FieldDataType = FieldDataType;
    params.IgnoreFilter = IgnoreFilter;
    params.IsCacheOnClient = IsCacheOnClient;
    params.ForceEnableAdd = ForceEnableAdd;
    this.pushAdditionalFilter(params);
  };

  _proto.myReplace = function myReplace(myString) {
    var myNewString = "";

    for (var i = 0; i < myString.length; i++) {
      if (myString[i] == '"') {
        myNewString = myNewString + '\\"';
      } else {
        myNewString = myNewString + myString[i];
      }
    }

    return myNewString;
  };

  _proto.removeAdditionalFilter = function removeAdditionalFilter(FieldName) {
    var item = this.AdditionalFilters.filter(function (d) {
      return d.FieldName == FieldName && d.IsLookUpfilter == true;
    })[0];

    if (item) {
      var index = this.AdditionalFilters.indexOf(item);
      this.AdditionalFilters.splice(index, 1);
    }
  };

  _proto.pushAdditionalFilter = function pushAdditionalFilter(params) {
    if (!params.IsCacheOnClient) {
      if (typeof params.FieldValue === "string") {
        if (params.FieldValue) params.FieldValue = this.myReplace(params.FieldValue);
        if (params.FieldName != "ImportersFilter") params.FieldValue = encodeURIComponent(params.FieldValue);
      }

      if (typeof params.FieldValue2 === "string") {
        if (params.FieldValue) params.FieldValue2 = this.myReplace(params.FieldValue2);
        params.FieldValue2 = encodeURIComponent(params.FieldValue2);
      }

      if (typeof params.FieldValue3 === "string") {
        if (params.FieldValue) params.FieldValue3 = this.myReplace(params.FieldValue3);
        params.FieldValue3 = encodeURIComponent(params.FieldValue3);
      }
    }

    var existedItem = this.AdditionalFilters.find(function (d) {
      return d.FieldName == params.FieldName;
    });

    if (!existedItem || params.ForceEnableAdd) {
      var item = new FilterItem(params.FieldName, params.FieldValue, params.FieldValue2, params.FieldValue3, params.Operator, params.IsCustom, params.DisplayInList, params.IsCustomField, params.FieldDataType, params.IgnoreFilter, params.IsCacheOnClient, params.IsLookUpFilter);
      this.AdditionalFilters.push(item);
    }
  };

  return ApiQueryFilters;
}();
var FilterItem = function FilterItem(FieldName, FieldValue, FieldValue2, FieldValue3, Operator, IsCustom, DisplayInList, IsCustomField, FieldDataType, IgnoreFilter, IsCacheOnClient, IsLookUpfilter) {
  if (IsCacheOnClient === void 0) {
    IsCacheOnClient = false;
  }

  if (IsLookUpfilter === void 0) {
    IsLookUpfilter = false;
  }

  this.FieldName = FieldName;
  this.FieldValue = FieldValue;
  this.FieldValue2 = FieldValue2;
  this.FieldValue3 = FieldValue3;
  this.Operator = Operator;
  this.IsCustom = IsCustom;
  this.DisplayInList = DisplayInList;
  this.IsCustomField = IsCustomField;
  this.FieldDataType = FieldDataType;
  this.IgnoreFilter = IgnoreFilter;
  this.IsCacheOnClient = IsCacheOnClient;
  this.IsLookUpfilter = IsLookUpfilter;
};
var ApiQueryFiltersAddParams = function ApiQueryFiltersAddParams() {
  this.FieldName = '';
  this.Operator = '';
  this.IsCustom = false;
  this.DisplayInList = false;
  this.IsCustomField = false;
  this.FieldDataType = '';
  this.IgnoreFilter = false;
  this.IsCacheOnClient = false;
  this.ForceEnableAdd = false;
  this.IsLookUpFilter = false;
};

var calcTypes = [{
  Name: "Sum",
  Disabled: false
}, {
  Name: "Min",
  Disabled: true
}, {
  Name: "Max",
  Disabled: false
}, {
  Name: "Count",
  Disabled: false
}, {
  Name: "Average",
  Disabled: true
}];
var chartState = {
  series: [{
    data: [10, 20, 30]
  }],
  options: {
    chart: {
      toolbar: {
        show: false
      }
    },
    colors: ["#22D792"],
    plotOptions: {
      bar: {
        borderRadius: 4,
        horizontal: true
      }
    },
    dataLabels: {
      enabled: false
    },
    xaxis: {
      categories: ['A', 'B', 'C']
    }
  }
};
var ChartTypes = [{
  Name: "Column",
  Disabled: true
}, {
  Name: "Area",
  Disabled: false
}, {
  Name: "Bar",
  Disabled: false
}, {
  Name: "Pie",
  Disabled: true
}, {
  Name: "Donut",
  Disabled: true
}];

var NewWidget = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(NewWidget, _React$Component);

  function NewWidget(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _this.onEntityChanged = function (e) {
      var selectedTable = e.target.value;
      _this.selectedTable = selectedTable;
      console.log("onEntityChanged", selectedTable);
      _this.state.Widget.EntityName = selectedTable === null || selectedTable === void 0 ? void 0 : selectedTable.Name;
      _this.state.Widget.EntityId = selectedTable === null || selectedTable === void 0 ? void 0 : selectedTable.Id;

      _this.setState({
        Widget: _extends({}, _this.state.Widget)
      }, function () {
        if (selectedTable) {
          _this.getObjectFields();
        } else {
          _this.state.Widget.Group = null;
          _this.state.Widget.ValueFieldId = null;
          _this.state.Widget.ValueFieldName = null;

          _this.setState({
            Widget: _extends({}, _this.state.Widget),
            objectFields: []
          });
        }
      });
    };

    _this.onGroupChanged = function (e) {
      _this.state.Widget.Group = e.value;

      _this.setState({
        Widget: _extends({}, _this.state.Widget)
      });
    };

    _this.onValueChanged = function (e) {
      _this.selectedValueField = e.value;
      _this.state.Widget.ValueFieldId = _this.selectedValueField.Id;
      _this.state.Widget.ValueFieldName = _this.selectedValueField.FieldName;

      _this.setState({
        Widget: _extends({}, _this.state.Widget)
      });
    };

    _this.onCalcTypeChanged = function (e) {
      _this.state.Widget.CalcType = e.value;

      _this.setState({
        Widget: _extends({}, _this.state.Widget)
      });
    };

    var defaultWidget = {
      WidgetName: 'New Widget',
      WidgetType: "Bar",
      CalcType: null,
      ValueFieldName: null,
      ValueFieldId: null,
      EntityName: null,
      EntityId: null,
      Group: null
    };
    _this.state = {
      Widget: _this.props.widget || defaultWidget,
      objectTables: [],
      objectFields: []
    };

    _this.getData();

    return _this;
  }

  var _proto = NewWidget.prototype;

  _proto.getData = function getData() {
    this.getObjectTables();
  };

  _proto.getObjectTables = function getObjectTables() {
    var _this2 = this;

    var service = new DataService();
    var filters = new ApiQueryFilters();
    filters.addAdditionalFilter("Name", 'GLAccount,ARPayment,ARInvoice', null, null, "InListExact", false, false, false, "string");
    service.getObjectTables(filters).then(function (response) {
      console.log("getObjectTables", response);

      if (response.data.Result && _this2.props.widget) {
        _this2.selectedTable = response.data.Result.find(function (t) {
          var _this2$props$widget;

          return t.Id === ((_this2$props$widget = _this2.props.widget) === null || _this2$props$widget === void 0 ? void 0 : _this2$props$widget.EntityId);
        });
      }

      _this2.setState({
        objectTables: response.data.Result
      });

      if (response.data.Result && _this2.props.widget) {
        _this2.getObjectFields();
      }
    }, function (error) {
      alert("Something wrong happen, " + error);
    });
  };

  _proto.getObjectFields = function getObjectFields() {
    var _this3 = this;

    var service = new DataService();
    var filters = new ApiQueryFilters();
    filters.SortBy = 'FieldName';
    filters.addAdditionalFilter("ObjectTableId", this.state.Widget.EntityId, null, null, "Equals", false, false, false, "string");
    service.getObjectFields(filters).then(function (response) {
      console.log("getByfilters", response);

      _this3.setState({
        objectFields: response.data.Result
      });
    }, function (error) {
      alert("Something wrong happen, " + error);
    });
  };

  _proto.render = function render() {
    var _this4 = this;

    return React__default.createElement("div", {
      className: "dialog-wrapper"
    }, React__default.createElement("div", {
      className: "dialog-content container"
    }, React__default.createElement("div", {
      style: {
        width: "250px",
        padding: '0 40px 0 0',
        textAlign: 'center'
      }
    }, React__default.createElement(Dropdown, {
      style: {
        width: "160px"
      },
      value: this.state.Widget.WidgetType,
      options: ChartTypes,
      placeholder: "Chart Type",
      optionLabel: "Name",
      optionValue: "Name",
      optionDisabled: function optionDisabled(option) {
        return option === null || option === void 0 ? void 0 : option.Disabled;
      },
      onChange: function onChange(e) {
        _this4.state.Widget.WidgetType = e.value;

        _this4.setState({
          Widget: _extends({}, _this4.state.Widget)
        });
      }
    }), React__default.createElement("br", null), React__default.createElement("br", null), React__default.createElement("div", null, this.state.Widget.WidgetType === 'Bar' && React__default.createElement(ReactApexChart, {
      options: chartState.options,
      series: chartState.series,
      type: "bar",
      height: "100%"
    }), this.state.Widget.WidgetType === 'Area' && React__default.createElement(ReactApexChart, {
      options: chartState.options,
      series: chartState.series,
      type: "area",
      height: "100%"
    }))), React__default.createElement("div", null, React__default.createElement("form", {
      id: "frm",
      name: "frm"
    }, React__default.createElement("div", {
      className: "form"
    }, React__default.createElement("div", {
      className: "form-field"
    }, React__default.createElement("label", {
      htmlFor: "widget-name"
    }, "Name"), React__default.createElement(InputText, {
      id: "widget-name",
      className: "p-inputtext-sm",
      value: this.state.Widget.WidgetName || undefined,
      required: true,
      onChange: function onChange(e) {
        _this4.state.Widget.WidgetName = e.target.value;

        _this4.setState({
          Widget: _extends({}, _this4.state.Widget)
        });
      }
    })), React__default.createElement("div", {
      className: "form-field"
    }, React__default.createElement("label", {
      htmlFor: "widget-Entity"
    }, "Entity"), React__default.createElement(Dropdown, {
      value: this.selectedTable,
      options: this.state.objectTables,
      onChange: this.onEntityChanged,
      optionLabel: "Name",
      filter: true,
      showClear: true,
      filterBy: "Name",
      placeholder: "Select an entity"
    })), React__default.createElement("div", {
      className: "form-field"
    }, React__default.createElement("label", {
      htmlFor: "widget-Entity"
    }, "Group"), React__default.createElement(Dropdown, {
      value: this.state.Widget.Group,
      options: this.state.objectFields,
      virtualScrollerOptions: {
        itemSize: 50
      },
      onChange: this.onGroupChanged,
      optionLabel: "FieldName",
      optionValue: "FieldName",
      filter: true,
      showClear: true,
      filterBy: "FieldName",
      placeholder: "Select group"
    })), React__default.createElement("div", {
      className: "form-field"
    }, React__default.createElement("label", {
      htmlFor: "widget-Entity"
    }, "Calc. Type"), React__default.createElement(Dropdown, {
      value: this.state.Widget.CalcType,
      options: calcTypes,
      optionLabel: "Name",
      optionValue: "Name",
      optionDisabled: function optionDisabled(option) {
        return option === null || option === void 0 ? void 0 : option.Disabled;
      },
      onChange: this.onCalcTypeChanged,
      showClear: true,
      required: true,
      placeholder: "Select type"
    })), React__default.createElement("div", {
      className: "form-field"
    }, React__default.createElement("label", {
      htmlFor: "widget-Entity"
    }, "Value"), React__default.createElement(Dropdown, {
      value: this.selectedValueField,
      options: this.state.objectFields,
      virtualScrollerOptions: {
        itemSize: 50
      },
      onChange: this.onValueChanged,
      optionLabel: "FieldName",
      filter: true,
      showClear: true,
      filterBy: "FieldName",
      placeholder: "Select value"
    })))))), React__default.createElement("div", {
      className: "dialog-footer"
    }, React__default.createElement(Button, {
      label: "Cancel",
      className: "p-button-text",
      onClick: this.props.onCancelButton.bind(this)
    }), React__default.createElement(Button, {
      label: "Ok",
      form: "frm",
      onClick: this.props.onOkButton.bind(this, _extends({}, this.state.Widget)),
      autoFocus: true
    })));
  };

  return NewWidget;
}(React__default.Component);

var ColumnChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(ColumnChartComponent, _React$Component);

  function ColumnChartComponent(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = ColumnChartComponent.prototype;

  _proto.render = function render() {
    return React__default.createElement("div", {
      id: "chart"
    });
  };

  return ColumnChartComponent;
}(React__default.Component);

var PieChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(PieChartComponent, _React$Component);

  function PieChartComponent(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = PieChartComponent.prototype;

  _proto.render = function render() {
    return React__default.createElement("div", null);
  };

  return PieChartComponent;
}(React__default.Component);

var MixedChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(MixedChartComponent, _React$Component);

  function MixedChartComponent(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = MixedChartComponent.prototype;

  _proto.render = function render() {
    return React__default.createElement("div", {
      id: "chart"
    });
  };

  return MixedChartComponent;
}(React__default.Component);

var AreaChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(AreaChartComponent, _React$Component);

  function AreaChartComponent(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {
      series: [{
        name: 'series1',
        data: [31, 40, 28, 51, 42, 109, 100]
      }, {
        name: 'series2',
        data: [11, 32, 45, 32, 34, 52, 41]
      }],
      options: {
        chart: {
          height: 350,
          type: 'area'
        },
        colors: ['#22D792', '#2C99F7', '#FBC445', '#F74E49', '#717585'],
        dataLabels: {
          enabled: false
        },
        stroke: {
          curve: 'smooth'
        },
        xaxis: {
          type: 'datetime',
          categories: ['2018-09-19T00:00:00.000Z', '2018-09-19T01:30:00.000Z', '2018-09-19T02:30:00.000Z', '2018-09-19T03:30:00.000Z', '2018-09-19T04:30:00.000Z', '2018-09-19T05:30:00.000Z', '2018-09-19T06:30:00.000Z']
        },
        tooltip: {
          x: {
            format: 'dd/MM/yy HH:mm'
          }
        }
      }
    };
    return _this;
  }

  var _proto = AreaChartComponent.prototype;

  _proto.render = function render() {
    return React__default.createElement("div", {
      id: "chart"
    }, this.props.dateFilter && React__default.createElement("div", {
      className: "filter-label"
    }, "  Date: ", this.props.dateFilter.toLocaleDateString("en-US") + ''), React__default.createElement(ReactApexChart, {
      options: this.state.options,
      series: this.state.series,
      type: "area",
      height: '100%'
    }));
  };

  return AreaChartComponent;
}(React__default.Component);

var TimelineChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(TimelineChartComponent, _React$Component);

  function TimelineChartComponent(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = TimelineChartComponent.prototype;

  _proto.render = function render() {
    return React__default.createElement("div", {
      id: "chart"
    });
  };

  return TimelineChartComponent;
}(React__default.Component);

var CandleChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(CandleChartComponent, _React$Component);

  function CandleChartComponent(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = CandleChartComponent.prototype;

  _proto.render = function render() {
    return React__default.createElement("div", {
      id: "chart"
    });
  };

  return CandleChartComponent;
}(React__default.Component);

var BarChartComponent = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(BarChartComponent, _React$Component);

  function BarChartComponent(props) {
    var _this$props$widget;

    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.data = [];
    var filters = new ApiQueryFilters();
    var dataService = new DataService();
    if ((_this$props$widget = _this.props.widget) !== null && _this$props$widget !== void 0 && _this$props$widget.EntityName) dataService.getDataByFilters(filters, _this.props.widget.EntityName || '').then(function (response) {
      console.log("dataService.getDataByFilters", response);
      _this.data = response.data.Result;

      _this.InitChart();
    });
    _this.state = {
      series: [{
        data: []
      }],
      options: {
        chart: {
          type: "bar",
          height: 350,
          events: {
            dataPointSelection: function dataPointSelection(event, chartContext, config) {
              var data = config.w.config.xaxis.categories[config.seriesIndex];
              console.log("Selected " + data);
            }
          }
        },
        colors: ["#22D792", "#2C99F7", "#FBC445", "#F74E49", "#717585"],
        plotOptions: {
          bar: {
            borderRadius: 4,
            horizontal: true
          }
        },
        dataLabels: {
          enabled: false
        },
        xaxis: {
          categories: []
        }
      }
    };
    return _this;
  }

  var _proto = BarChartComponent.prototype;

  _proto.InitChart = function InitChart() {
    var groups = this.GroupItems();
    var catItems = Object.keys(groups);
    var dataItems = Object.values(groups);
    this.setState({
      series: [{
        data: dataItems
      }],
      options: {
        chart: {
          type: "bar",
          height: 350,
          events: {
            dataPointSelection: function dataPointSelection(event, chartContext, config) {
              var data = config.w.config.xaxis.categories[config.seriesIndex];
              console.log("Selected " + data);
            }
          }
        },
        colors: ["#22D792", "#2C99F7", "#FBC445", "#F74E49", "#717585"],
        plotOptions: {
          bar: {
            borderRadius: 4,
            horizontal: true
          }
        },
        dataLabels: {
          enabled: false
        },
        xaxis: {
          categories: catItems
        }
      }
    });
  };

  _proto.GroupItems = function GroupItems() {
    var _this2 = this;

    var groups = this.data.reduce(function (groups, record) {
      var CreateGroup = function CreateGroup(groups, record) {
        var group = groups[record] || [];
        group.push(record);
      };

      var SetGroupData = function SetGroupData() {
        var groupByFieldName = _this2.props.widget.Group + "";
        var valueFieldName = _this2.props.widget.ValueFieldName + "";
        var recordValue = record[valueFieldName];
        var currentGroupValue = groups[record[groupByFieldName]];

        switch (_this2.props.widget.CalcType) {
          case "Count":
            {
              if (!currentGroupValue) currentGroupValue = 0;
              currentGroupValue = currentGroupValue + 1;
              break;
            }

          case "Sum":
            {
              if (!currentGroupValue) currentGroupValue = 0;
              currentGroupValue = currentGroupValue + recordValue;
              break;
            }

          case "Max":
            {
              if (!currentGroupValue) currentGroupValue = -9999999999999;
              if (recordValue > currentGroupValue) currentGroupValue = recordValue;
              break;
            }

          case "Min":
            {
              if (!currentGroupValue) currentGroupValue = 9999999999999;
              if (recordValue < currentGroupValue) currentGroupValue = recordValue;
              break;
            }

          case "Average":
            {
              if (!currentGroupValue) currentGroupValue = [];
              currentGroupValue.push(recordValue);
              break;
            }
        }

        groups[record[groupByFieldName]] = currentGroupValue;
      };

      CreateGroup(groups, record);
      SetGroupData();
      return groups;
    }, {});

    if (this.props.widget.CalcType == "Average") {
      var _loop = function _loop(key) {
        if (Object.prototype.hasOwnProperty.call(groups, key)) {
          var groupArray = groups[key];
          var sum = groupArray.reduce(function (a, b) {
            return a + (b[key] || 0);
          }, 0);
          var avg = sum / groupArray.length;
          groups[key] = avg;
        }
      };

      for (var key in groups) {
        _loop(key);
      }
    }

    return groups;
  };

  _proto.render = function render() {
    return React__default.createElement("div", {
      id: "chart"
    }, React__default.createElement("div", null, this.props.dateFilter && React__default.createElement("div", {
      className: "filter-label"
    }, "  Date: ", this.props.dateFilter.toLocaleDateString("en-US") + '')), React__default.createElement(ReactApexChart, {
      options: this.state.options,
      series: this.state.series,
      type: "bar",
      height: "100%"
    }));
  };

  return BarChartComponent;
}(React__default.Component);

var WidgetCard = function WidgetCard(props, state) {
  var _props$widget, _widgetComponents$fin;

  var moreBtnToggle = useRef(null);
  var widgetComponents = [{
    type: "Bar",
    component: BarChartComponent
  }, {
    type: "Column",
    component: ColumnChartComponent
  }, {
    type: "Pie",
    component: PieChartComponent
  }, {
    type: "Mixed",
    component: MixedChartComponent
  }, {
    type: "Timeline",
    component: TimelineChartComponent
  }, {
    type: "Candle",
    component: CandleChartComponent
  }, {
    type: "Area",
    component: AreaChartComponent
  }];
  return React__default.createElement("div", {
    className: "panel",
    key: props.el.i
  }, React__default.createElement("header", null, React__default.createElement("h1", null, ((_props$widget = props.widget) === null || _props$widget === void 0 ? void 0 : _props$widget.WidgetName) || "not set"), React__default.createElement(Button, {
    icon: "pi pi-ellipsis-v",
    className: "p-button-lg p-button-rounded p-button-text",
    onClick: function onClick(e) {
      var _moreBtnToggle$curren;

      return (_moreBtnToggle$curren = moreBtnToggle.current) === null || _moreBtnToggle$curren === void 0 ? void 0 : _moreBtnToggle$curren.toggle(e);
    }
  }), React__default.createElement(OverlayPanel, {
    ref: moreBtnToggle,
    className: "buttons-overlay"
  }, React__default.createElement("div", {
    className: "buttons-list"
  }, React__default.createElement("div", {
    onClick: function onClick() {
      return props.editBtnClicked(props.widget);
    }
  }, " Edit "), React__default.createElement("div", {
    onClick: function onClick() {
      return props.deleteBtnClicked(props.el.i);
    }
  }, "Remove")))), React__default.createElement("div", {
    className: "dashboard-widget"
  }, React__default.createElement((_widgetComponents$fin = widgetComponents.find(function (wc) {
    return wc.type === props.widget.WidgetType;
  })) === null || _widgetComponents$fin === void 0 ? void 0 : _widgetComponents$fin.component, {
    key: props.el.i,
    dateFilter: props.dateFilter,
    widget: props.widget
  })));
};

var ResponsiveReactGridLayout = WidthProvider(Responsive);
var newWidgetWidth = 3;

var DashboardDesigner = function DashboardDesigner() {
  var _useState = useState(createRef());

  var _useState2 = useState(createRef());

  var _useState3 = useState([{
    type: "Bar",
    component: BarChartComponent
  }, {
    type: "Column",
    component: ColumnChartComponent
  }, {
    type: "Pie",
    component: PieChartComponent
  }, {
    type: "Mixed",
    component: MixedChartComponent
  }, {
    type: "Timeline",
    component: TimelineChartComponent
  }, {
    type: "Candle",
    component: CandleChartComponent
  }, {
    type: "Area",
    component: AreaChartComponent
  }]),
      widgetComponents = _useState3[0];

  var savedLayouts = getFromStorage("grid-layout");
  var savedWidgets = getFromStorage("grid-widgets");

  var _useState4 = useState(savedLayouts || {
    lg: []
  }),
      layouts = _useState4[0],
      setLayouts = _useState4[1];

  var _useState5 = useState(false),
      setShowMsg = _useState5[1];

  var _useState6 = useState(undefined),
      dateFilter = _useState6[0];

  var _useState7 = useState(false),
      displayDialog = _useState7[0],
      setDisplayDialog = _useState7[1];

  var _useState8 = useState("");

  var _useState9 = useState(savedWidgets || []),
      widgets = _useState9[0],
      setWidgets = _useState9[1];

  var _useState10 = useState(10),
      placeholderRows = _useState10[0],
      setPlaceholderRows = _useState10[1];

  var _useState11 = useState(),
      selectedWidget = _useState11[0];

  var _useState12 = useState(null),
      widgetToEdit = _useState12[0],
      setWidgetToEdit = _useState12[1];

  var renderPageHeader = function renderPageHeader() {
    return React__default.createElement("header", {
      className: "dl-dashboard-header"
    }, React__default.createElement("div", {
      className: "dl-heder-row"
    }, React__default.createElement("div", {
      style: {
        width: '192px'
      }
    }, React__default.createElement(Dropdown, {
      value: selectedWidget,
      options: WidgetTypes,
      optionValue: "Name",
      optionLabel: "Name",
      onChange: function onChange(e) {
        return addComponent();
      },
      placeholder: "Add Component",
      optionDisabled: function optionDisabled(option) {
        return option === null || option === void 0 ? void 0 : option.Disabled;
      },
      scrollHeight: "300px"
    }))));
  };

  function addComponent(componentType) {
    toggleDialog();
  }

  function renderGridWidgetsPlaceholder() {
    console.log(placeholderRows);
    return React__default.createElement("div", {
      className: "dl-grid dl-grid--bordered"
    }, [].concat(Array(placeholderRows)).map(function (e, i) {
      return [].concat(Array(layoutGridProps.cols.lg)).map(function (e, i) {
        return React__default.createElement("div", {
          style: {
            height: layoutGridProps.rowHeight
          }
        });
      });
    }));
  }

  function renderLayoutGrid(layout) {
    return React__default.createElement(ResponsiveReactGridLayout, Object.assign({}, layoutGridProps, {
      layouts: layouts,
      compactType: "vertical",
      onLayoutChange: function onLayoutChange(layout, layouts) {
        return _onLayoutChange(layout, layouts);
      }
    }), _.map(layout, function (el) {
      return renderWidgetComponent(el);
    }));
  }

  function _onLayoutChange(layout, layouts) {
    console.log(layout, layouts);
    saveToStorage("grid-layout", layouts);
    saveToStorage("grid-widgets", widgets);
    setLayouts(layouts);
  }

  function renderWidgetComponent(el) {
    var widget = FindWidget(el);
    return React__default.createElement("div", {
      key: el.i
    }, React__default.createElement(WidgetCard, {
      key: el.i,
      widget: widget,
      el: el,
      dateFilter: dateFilter,
      editBtnClicked: editWidget,
      deleteBtnClicked: deletePanel
    }));
  }

  function deletePanel(i) {
    var layout = layouts["lg"];
    var itemIndex = layout.findIndex(function (d) {
      return d.i === i;
    });
    layout.splice(itemIndex, 1);
    setLayouts({
      lg: [].concat(layout)
    });
    setWidgets(_.reject(widgets, {
      i: i
    }));
    SaveGrid();
  }

  function SaveGrid() {
    saveToStorage("grid-layout", layouts);
    saveToStorage("grid-widgets", widgets);
  }

  function editWidget(widget) {
    setWidgetToEdit(widget);
    toggleDialog();
  }

  function FindWidget(el) {
    return widgets.find(function (d) {
      return d.i === el.i;
    });
  }

  function saveToStorage(key, value) {
    setShowMsg(true);
    setTimeout(function () {
      setShowMsg(false);
    }, 2000);
    window.localStorage.setItem(key, JSON.stringify(value));
  }

  function getFromStorage(key) {
    var layout = window.localStorage.getItem(key);
    return JSON.parse(layout);
  }

  function toggleDialog() {
    setDisplayDialog(!displayDialog);
  }

  function renderNewWidgetDialog() {
    return React__default.createElement(Dialog, {
      header: widgetToEdit ? "Edit Chart" : "New Chart",
      visible: displayDialog,
      onHide: toggleDialog,
      breakpoints: {
        "850px": "95vw",
        "600px": "100vw"
      },
      style: {
        width: "800px"
      }
    }, React__default.createElement(NewWidget, {
      onOkButton: onOkButton,
      onCancelButton: onCancelButton,
      widget: widgetToEdit
    }));
  }

  function addPanel(newWidget) {
    var newWidgetPosition = EvaluateNewWidgetPosition();
    var widgetComponent = widgetComponents.find(function (wc) {
      return wc.type === newWidget.WidgetType;
    });
    var randomIndex = getRandomInt(100, 999);
    var newPanel = {
      i: "" + randomIndex,
      x: newWidgetPosition.x,
      y: newWidgetPosition.y,
      w: newWidgetWidth,
      h: 5,
      minH: 5,
      minW: 3
    };
    widgets.push(_extends({
      i: "" + randomIndex,
      title: newWidget.WidgetName,
      component: widgetComponent === null || widgetComponent === void 0 ? void 0 : widgetComponent.component
    }, newWidget));
    setLayouts({
      lg: layouts["lg"].concat(newPanel)
    });
    setWidgets(widgets);
  }

  function EvaluateNewWidgetPosition() {
    var widgetYPosition = 0;
    var widgetXPosition = 0;
    var layout = layouts["lg"];
    var maxY = layout.reduce(function (max, widget) {
      if (widget.y > max) max = widget.y;
      return max;
    }, 0);

    var _loop = function _loop(y) {
      var widgetsRow = layout.filter(function (d) {
        return d.y == y;
      });
      var lastWidgetStartXPosition = Math.max.apply(Math, widgetsRow.map(function (a) {
        return a.x;
      }));
      var lastWidget = widgetsRow.find(function (a) {
        return a.x == lastWidgetStartXPosition;
      });

      if (lastWidget) {
        var lastWidgetEndXPosition = lastWidgetStartXPosition + lastWidget.w;
        var availableHorizontalSlots = layoutGridProps.cols.lg - lastWidgetEndXPosition;

        if (availableHorizontalSlots >= newWidgetWidth) {
          widgetYPosition = y;
          widgetXPosition = lastWidgetEndXPosition;
          return {
            v: {
              x: widgetXPosition,
              y: widgetYPosition
            }
          };
        } else {
          widgetYPosition = y + 1;
        }
      } else {
        widgetYPosition = y + 1;
      }
    };

    for (var y = 0; y <= maxY; y++) {
      var _ret = _loop(y);

      if (typeof _ret === "object") return _ret.v;
    }

    return {
      x: widgetXPosition,
      y: widgetYPosition
    };
  }

  function onOkButton(data) {
    setWidgetToEdit(null);
    toggleDialog();
    addPanel(data);
  }

  function onCancelButton() {
    setWidgetToEdit(null);
    toggleDialog();
  }

  function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
  }

  useEffect(function () {
    UpdatePlaceholderDimensions();
  });

  function UpdatePlaceholderDimensions() {
    var _gridLayout$;

    var minimumPlaceholderRowsCount = 10;
    var gridLayout = document.getElementsByClassName('react-grid-layout');
    if (!gridLayout) return;
    var gridElementHeight = (_gridLayout$ = gridLayout[0]) === null || _gridLayout$ === void 0 ? void 0 : _gridLayout$.clientHeight;
    var newPlaceholderRows = Math.floor(gridElementHeight / layoutGridProps.rowHeight);
    newPlaceholderRows = newPlaceholderRows < minimumPlaceholderRowsCount ? 10 : newPlaceholderRows;
    setPlaceholderRows(newPlaceholderRows);
  }

  return React__default.createElement("div", {
    className: "dl-full-hight"
  }, React__default.createElement("div", {
    className: "dl-box"
  }, React__default.createElement("div", {
    className: "dl-row dl-header"
  }, renderPageHeader()), React__default.createElement("div", {
    className: "dl-row dl-content"
  }, React__default.createElement("div", {
    className: "dl-grid-container"
  }, renderGridWidgetsPlaceholder(), renderLayoutGrid(layouts["lg"])))), renderNewWidgetDialog());
};

var layoutGridProps = {
  rowHeight: 50,
  className: "layout",
  cols: {
    lg: 12,
    md: 6,
    sm: 2,
    xs: 1,
    xxs: 1
  },
  useCSSTransforms: true
};
var WidgetTypes = [{
  Name: "Filter",
  Disabled: true
}, {
  Name: "Chart",
  Disabled: false
}, {
  Name: "Data Table",
  Disabled: true
}, {
  Name: "Pivot Table",
  Disabled: true
}, {
  Name: "KPI",
  Disabled: true
}, {
  Name: "Gauge",
  Disabled: true
}];

var Dashboard = function Dashboard(props) {
  localStorage.setItem("token", props === null || props === void 0 ? void 0 : props.token);
  return createElement("div", {
    className: 'dl-dashboard-body'
  }, createElement(DashboardDesigner, null));
};

export default Dashboard;
//# sourceMappingURL=index.modern.js.map
